require_relative 'base.rb'
require_relative 'constants.rb'

module Nastika

# This class provides way to talk to the host over UDP.
class UDP
  attr_accessor :host
  attr_accessor :port
  attr_accessor :payload
  attr_accessor :connected
  attr_accessor :connection

  # Initializes the Nastika::UDP object. 
  # WARNING: Does not connect to host automatically.
  # Params:
  # +host+:: Hostname of the target.
  # +port+:: Port on host to connect to.
  # +payload+:: The payload to send to host.
  def initialize(host=nil, port=nil, payload=Nastika::Constants::DEFAULT_PAYLOAD)
    self.host = host
    self.port = port
    self.payload = payload
    self.connected = false

    unless self.host.nil? & self.port.nil?
      self.connection = Base.new(Nastika::Constants::UDP)
      self.connection.host = self.host
      self.connection.port = self.port
      self.connection.payload = payload
    end
  end
  
  # Connects to the host over UDP
  def connect
    if ! self.connected
      begin 
        self.connection.connect
        self.connected = true
        self.connection
      rescue SocketError
        raise ArgumentError, "Cannot connect to #{self.host}:#{self.port}"
      end
    else
      raise RuntimeError, "We're already connected."
    end
  end

  # Sends the payload to the host and returns the read bytes.
  # If a payload is passed as argument, send that else sends the 
  # payload set during initialization.
  # Params:
  # +payload+:: Default is "GET / HTTP/1.0\r\n\r\n" or whatever was initialized.
  # +bytes+:: Amount of byted to read from host after sending request. 
  def send(paylaod=self.payload, bytes=Nastika::Constants::DEFAULT_BYTES)
    self.connection.send(payload)
    self.connection.get(Nastika::Constants::DEFAULT_BYTES)
  end

  # Closes the connection
  def close 
    if self.connected
      self.connection.quit
      self.connected = false
    else
      raise RuntimeError, "We're not connected yet."
    end
  end
end

end